<?php

namespace Fastrack\Seo;

use PhpParser\Node\Scalar\MagicConst\Dir;
use PlatoCreative\Imgix\Imgix;
use SilverStripe\AssetAdmin\Forms\UploadField;
use SilverStripe\Control\Director;
use SilverStripe\Control\HTTPRequest;
use SilverStripe\Forms\CheckboxField;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\HeaderField;
use SilverStripe\Forms\TextareaField;
use SilverStripe\Forms\TextField;
use SilverStripe\Forms\ToggleCompositeField;
use SilverStripe\ORM\DataExtension;
use SilverStripe\SiteConfig\SiteConfig;
use SilverStripe\Subsites\Model\Subsite;
use SilverStripe\View\Requirements;
use SwiftDevLabs\CodeEditorField\Forms\CodeEditorField;

class SeoExtension extends DataExtension
{

	private static $db = [
		'MetaTitle' => 'Varchar(255)',
		'DoNotShowSiteTitle' => 'Boolean',
		'MetaDescription' => 'Text',
		'ShowInSiteMap' => 'Boolean',
		'TwitterSite' => 'Varchar(255)',
		'TwitterTitle' => 'Varchar(255)',
		'TwitterDescription' => 'Text',
		'TwitterImageAltTag' => 'Varchar(255)',
		'OpenGraphTitle' => 'Varchar(255)',
		'OpenGraphType' => 'Varchar(255)',
		'JavaScriptHead' => 'Text',
		'JavaScriptBodyTop' => 'Text',
		'JavaScriptBodyBottom' => 'Text',
	];

	private static $has_one = [
		'TwitterImage' => Imgix::class,
		'OpenGraphImage' => Imgix::class,
	];

	private static $translate = [
		'MetaTitle',
		'MetaDescription',
	];

	public function updateCMSFields(FieldList $fields)
	{
		Requirements::set_force_js_to_bottom(true);
		foreach (self::$db as $key => $value) {
			$fields->removeByName($key);
		}
		foreach (self::$has_one as $key => $value) {
			$fields->removeByName($key);
		}

		$mode = 'ace/mode/html';
		$fields->addFieldsToTab('Root.Main', [
			HeaderField::create('SeoTitle', 'SEO'),
			new TextField('MetaTitle'),
			new CheckboxField('DoNotShowSiteTitle'),
			new TextareaField('MetaDescription'),
			ToggleCompositeField::create('Twitter', 'Twitter', [
				new TextField('TwitterSite'),
				new TextField('TwitterTitle'),
				new TextareaField('TwitterDescription'),
				new UploadField('TwitterImage'),
				new TextField('TwitterImageAltTag'),
			]),
			ToggleCompositeField::create('OpenGraph', 'OpenGraph', [
				new TextField('OpenGraphTitle', 'OpenGraph Title'),
				new TextField('OpenGraphType', 'OpenGraph Type'),
				new UploadField('OpenGraphImage')
			]),
			ToggleCompositeField::create('CustomHTML', 'Custom HTML', [
				HeaderField::create('JavaScriptTitle', 'On page custom HTML and JavaScript'),
				CodeEditorField::create('JavaScriptHead', 'HTML inside <head> tag')->setMode($mode),
				CodeEditorField::create('JavaScriptBodyTop', 'HTML after the <body> tag')->setMode($mode),
				CodeEditorField::create('JavaScriptBodyBottom', 'HTML before </body> tag')->setMode($mode),
			]),
		]);
	}

	public function onBeforeWrite()
	{
		if (!$this->owner->ID) {
			$this->owner->ShowInSiteMap = true;
		}
	}


	public function generateMetaTitle()
	{
		return $this->getTitle('MetaTitle');
	}

	public function generateMetaDescription()
	{
		return $this->owner->MetaDescription;
	}

	public function generateTwitterSite()
	{
		return ($this->owner->TwitterSite) ? $this->owner->TwitterSite : false;
	}

	public function generateTwitterTitle()
	{
		return $this->getTitle('TwitterTitle');
	}

	public function generateTwitterDescription()
	{
		if ($this->owner->TwitterDescription) {
			return $this->owner->TwitterDescription;
		} else if ($this->owner->MetaDescription) {
			return $this->owner->MetaDescription;
		}
		return false;
	}

	public function generateTwitterImage()
	{
		$image = $this->getImage('TwitterImage');
		if ($image && $image->exists()) {
			return $image->Fit(255, 255)->Link();
		}
		return false;
	}

	public function generateTwitterImageAltTag()
	{
		$image = $this->getImage('TwitterImage');
		if ($image) {
			return ($image->Title) ?: false;
		}
		return false;
	}

	public function generateOpenGraphTitle()
	{
		return $this->getTitle('OpenGraphTitle');
	}

	public function generateOpenGraphType()
	{
		return 'website';
	}

	public function generateOpenGraphImage()
	{
		$image = $this->getImage('OpenGraphImage');
		if ($image && $image->exists()) {
			return $image->Fit(255, 255)->Link();
		}
		return false;
	}

	public function generateCanonical()
	{
		$subsite = Subsite::currentSubsite();
		if ($subsite) {
			$canonical = $this->getSubsiteCanonical($subsite);
		} else {
			$canonical = $this->getMainSiteCanonical();
		}
		return rtrim($canonical, '/') . '/';
	}


	private function getTitle($field)
	{
		$title = '';
		if ($this->owner->$field) {
			$title = $this->owner->$field;
		} else if ($this->owner->Title) {
			$title = $this->owner->Title;
		} else if ($this->owner->Name) {
			$title = $this->owner->Name;
		}
		if (!$this->owner->DoNotShowSiteTitle) {
			$siteConfig = SiteConfig::get()->first();
			switch ($siteConfig->SiteTitleUsage) {
				case 'As prefix':
					$title = $siteConfig->Title . ' ' . $siteConfig->TitleSeparator . ' ' . $title;
					break;
				case 'As postfix':
					$title .= ' ' . $siteConfig->TitleSeparator . ' ' . $siteConfig->Title;
					break;
			}
		}
		return $title;
	}

	private function getImage($field)
	{
		if ($this->owner->$field) {
			return $image = $this->owner->$field;
		}
		return false;
	}

	private function getSubsiteCanonical(Subsite $subsite)
	{
		return $subsite->getPrimarySubsiteDomain()->Link() . $_SERVER['REQUEST_URI'];
	}

	private function getMainSiteCanonical()
	{
		$siteConfig = SiteConfig::get()->first();
		$protocol = ($siteConfig->IsLive) ? 'https://' : 'http://';
		$domain = ($siteConfig->PrimaryDomain) ?: $_SERVER['HTTP_HOST'];
		return $protocol . $domain . $_SERVER['REQUEST_URI'];
	}




}
